<?php

/**
 * Release numbers
 */
class Less_Version {

	public const version = '3.2.1';			// The current build number of less.php
	public const less_version = '2.5.3';		// The less.js version that this build should be compatible with
	public const cache_version = '253';		// The parser cache version

}
